/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2016                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/*! \file trimesh_base.cpp
\ingroup code_sample

\brief the minimal example of using the lib

This file contain a minimal example of the library, showing how to load a mesh and how to compute per vertex normals on it.

*/

#include<vcg/complex/complex.h>
#include<wrap/io_trimesh/import_off.h>
#include<wrap/io_trimesh/export_off.h>
#include<vcg/complex/algorithms/create/platonic.h>


class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public vcg::UsedTypes<vcg::Use<MyVertex>   ::AsVertexType,
                                           vcg::Use<MyEdge>     ::AsEdgeType,
                                           vcg::Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Normal3f, vcg::vertex::VFAdj, vcg::vertex::BitFlags  >{};
class MyFace    : public vcg::Face<   MyUsedTypes, vcg::face::FFAdj, vcg::face::VFAdj, vcg::face::VertexRef, vcg::face::Color4b, vcg::face::BitFlags > {};
class MyEdge    : public vcg::Edge<   MyUsedTypes> {};

class MyMesh    : public vcg::tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace> , std::vector<MyEdge>  > {};

class MyVertex0  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::BitFlags  >{};
class MyVertex1  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Normal3f, vcg::vertex::BitFlags  >{};
class MyVertex2  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Color4b, vcg::vertex::CurvatureDirf,
                                                    vcg::vertex::Qualityf, vcg::vertex::Normal3f, vcg::vertex::BitFlags  >{};
using namespace vcg;

    

int main( int argc, char **argv )
{
  if(argc<2)
  {
    printf("Usage trimesh_base <meshfilename.off>\n");
    return -1;
  }
  /*!
    */
  MyMesh m;
  tri::io::ImporterOFF<MyMesh>::Open(m,argv[1]);
  printf("Input mesh '%s'  vn:%i fn:%i\n",argv[1],m.VN(),m.FN());
  
  tri::RequirePerVertexNormal(m);
  tri::UpdateTopology<MyMesh>::FaceFace(m);
  tri::UpdateFlags<MyMesh>::FaceClearV(m);
  
  // if there are no borders the number of edges should be
  // face_number * 3 / 2
  int faceBorderCnt=0;
  int loopCnt=0;
  for(MyMesh::FaceIterator fi=m.face.begin();fi!=m.face.end();++fi)
  {
      for(int i=0;i<3;++i)
          // face::IsBorder(f,e) is just f.FFp(e) == &f
          if(face::IsBorder(*fi,i) && !fi->IsV()){              
              printf("Starting visit of border %i\n",loopCnt);
              loopCnt++;
              face::Pos<MyFace> startPos(&*fi,i,fi->V(i));
              face::Pos<MyFace> runPos=startPos;
              assert(runPos.IsBorder());
              int faceLoopCnt=0;
              do
              {
                  runPos.F()->SetV();
                  runPos.F()->C()=Color4b::Red;
                  faceLoopCnt++;
                  faceBorderCnt++;
                  runPos.FlipE();
                  while(!runPos.IsBorder()) {
                      runPos.FlipF();
                      runPos.FlipE();
                  }
                  runPos.FlipV();
              } while(runPos != startPos);
              printf("ended Loop with %i border faces\n",faceLoopCnt);
          }
  }
  if(faceBorderCnt==0)
  {
      int en = (m.FN()*3) / 2;
      printf("Euler Characteristic == %i - %i + %i = %i\n",m.VN(), en, m.FN(), m.VN() - en + m.FN());
  }
  else {
      printf("Mesh has %i border edges and %i border loops\n",faceBorderCnt, loopCnt);
      
      int en = (m.FN()*3 + faceBorderCnt) / 2;
      printf("Euler Characteristic == %i - %i + %i = %i\n",m.VN(), en, m.FN(), m.VN() - en + m.FN());
  }
  
  tri::io::ExporterOFF<MyMesh>::Save(m,"Test.off",tri::io::Mask::IOM_FACECOLOR);
  
  return 0;
}
